Program TestMHZ;

{ Copyright (C) 1999 Idan Nof (GNU GPL) }
{ Read the attached files "README.TXT" and/or "GPL.TXT" for more details }

{$N+,E-,G+,B-,D-,I-,L-,Q-,R-,S-,V-,S-}

{$DEFINE USETSC}   { Choose the method for reading the TSC }

Uses TPTimer;

Var
  Start    : LongInt;
  Clocks   : Comp;
  Supported: Byte;
  N, Err   : Integer;

{----------------------------------------------------------------------------}
Function WinRunning :Boolean; Assembler;
Asm
  MOV  AX,$4680                       {  Win 3.x Standard check       }
  INT  $2F                            {  Call Int 2F                  }
  CMP  AX,0                           {  IF AX = 0 Win in real mode   }
  JNE  @EnhancedCheck                 {  If not check for enhanced mode}
  MOV  AL,1                           {  Set Result to true           }
  JMP  @Exit                          {  Go to end of routine         }
@EnhancedCheck:                       {  Else check for enhanced mode }
  MOV  AX,$1600                       {  Win 3.x Enhanced check       }
  INT  $2F                            {  Call Int 2F                  }
  CMP  AL,0                           {  Check returned value         }
  JE   @False                         {  If not one of the below it   }
  CMP  AL,$80                         {  is NOT installed             }
  JE   @False
  MOV  AL,1                           {  Nope it must BE INSTALLED    }
  JMP  @Exit
@False:
  MOV  AL,0                           {  Set Result to False          }
@Exit:
End;{WinRunning}

{----------------------------------------------------------------------------}
Procedure CheckSupported; Assembler;
Asm
    { Do we have a 486 or a Pentium? }
    { Does pushf/popf preserve the CPUID bit? (486=no, P5=yes) }
             inc    Supported
    db 66h;  mov    AX, CX        { get original EFLAGS}
    db 66h, 35h; dd $200000       { XOR id bit in flags}
    db 66h;  push   AX
    db 66h;  popf
    db 66h;  pushf
    db 66h;  pop    AX
    db 66h;  xor    AX, CX        { Is CPUID bit toggled? }
             je     @@1           { if not, we have an old 486 }
             inc    Supported
    db 66h;  xor    AX, AX
    db $f,$a2                     { CPUID, AX = 0 (get CPUID caps) }
    db 66h;  cmp    AX, 1
             jl     @@1           { if < 1, then exit }
             inc    Supported
    db 66h;  xor    AX, AX
    db 66h;  inc    AX
    db $f,$a2                     { CPUID, AX = 1 (get CPU info)   }
             test   DL, 00001000b { mask out all but bit 4 (TSC available)}
             jz     @@1
             inc    Supported
   @@1:
  end;

{----------------------------------------------------------------------------}
procedure ClockOn; assembler;
asm
{$IFDEF USETSC}
  db 0Fh, 31h                      (* rdtsc                      *)
{$ELSE}
  db 66h; mov cx,0010h; dw 0000h   (* mov ecx,10h (TSC)          *)
  db 0Fh, 32h                      (* rdmsr                      *)
{$ENDIF}

  db 66h; mov word ptr Clocks,ax   (* mov dword ptr Clocks,eax   *)
  db 66h; mov word ptr Clocks+4,dx (* mov dword ptr Clocks+4,edx *)
end;

{----------------------------------------------------------------------------}
procedure ClockOff; assembler;
asm
{$IFDEF USETSC}
  db 0Fh, 31h                      (* rdtsc                      *)
{$ELSE}
  db 66h; mov cx,0010h; dw 0000h   (* mov ecx,10h (TSC)          *)
  db 0Fh, 32h                      (* rdmsr                      *)
{$ENDIF}

  db 66h; sub ax,word ptr Clocks   (* sub eax,dword ptr Clocks   *)
  db 66h; sbb dx,word ptr Clocks+4 (* sbb edx,dword ptr Clocks+4 *)
  db 66h; mov word ptr Clocks,ax   (* mov dword ptr Clocks,eax   *)
  db 66h; mov word ptr Clocks+4,dx (* mov dword ptr Clocks+4,edx *)
end;

{----------------------------------------------------------------------------}
Begin
  Writeln('TestMHZ  Version 0.99  Copyright (C) 1999 Idan Nof');
  If WinRunning then
     Begin
        Writeln;
        Writeln('Warning : Windows detected - multitaskers aren''t recommended !');
        Write('Press <Ctrl+C> to quit now or <Enter> to continue.');
        Readln;
     End;

  N := 0;
  If ParamCount > 0 then Val(ParamStr(1),N,Err);
  If Not (N in [0..6]) then N := 0;

  Writeln;
  Supported:=0;
  If Test8086>=2 {At least 386} then CheckSupported;
  If Supported<4 then
         Begin
             Write('Need a');
             Case Supported of
                  0: Write(' Pentium');
                  1: Write(' CPUID');
                  2: Write(' CPUID capabilities function');
                  3: Write(' TSC (Time Stamp Counter)');
             End;
             Writeln('-compatible system to run this program.');
             Halt(1);
         End;

  Write('Please wait a sec...');

  Start:=ReadTimer;      { Save Start Time }
  ClockOn;               { Save Start TSC }
  While ElapsedTime(Start,ReadTimer) < 1000 do; {Delays 1000MS}
  ClockOff;              { Subtract : Clocks <-- EndTSC - StartTSC }
  Writeln;
  Writeln('Your CPU has a clock frequency of '  { Freq(Hz)/10^6 = Freq(MHz) }
          ,(Clocks/1000000):0:n,' MHz.');
End.
